#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#define MAX_INPUT 10 * 1024
#define MAX_TAB 8
// MAX_TAB should be at least 8
#define MAX_OUTPUT ( MAX_INPUT * MAX_TAB + 16 )

const char *lpszAppName = "ml-ent v0.1";

void HtmlEntities(LPSTR lpInput, LPSTR lpOutput);
void Info(LPSTR msg);
int InitializeConvParams(LPSTR lpszQuot, LPSTR lpszApos, LPSTR lpszBr, LPSTR lpszTab);
int ParseLine( char *buf, char *szVariable );

int  WINAPI WinMain(HINSTANCE hThis, HINSTANCE hPrev,
	LPSTR lpszArgs, int nMode)
{
	OpenClipboard(NULL);

	HANDLE hClippedText = GetClipboardData( CF_TEXT );

		if( hClippedText == NULL ) {
			CloseClipboard();
			Info("No text found in Clipboard");
			return 1;
		}

	LPSTR lpszClippedText = (LPSTR)GlobalLock( hClippedText );

		if( lpszClippedText == NULL ) { // abnormal
			Info("!ClobalLock");
			GlobalUnlock( hClippedText );
			CloseClipboard();
			return 1;
    	}

		if( lstrlen( lpszClippedText ) > MAX_INPUT ) {
           	Info("Too long text in Clipboard");
            GlobalUnlock( hClippedText );
       	    CloseClipboard();
            return 1;
        }

    char lpszInput[ MAX_INPUT + 1 ];

	lstrcpy( lpszInput, lpszClippedText );

	GlobalUnlock( hClippedText );

    CloseClipboard();

	HGLOBAL hNewText = GlobalAlloc( GMEM_DDESHARE, MAX_OUTPUT + 1 );

		if( hNewText == NULL ) { // abnormal
        	Info("Not enough memory!");
            return 1;
        }

	LPSTR lpszOutput = (LPSTR)GlobalLock( hNewText );

	HtmlEntities( lpszInput, lpszOutput );

    OpenClipboard( NULL );
    EmptyClipboard();
	
	GlobalUnlock( hNewText );
    
	SetClipboardData(CF_TEXT, hNewText );
    CloseClipboard();

//	GlobalUnlock( hNewText );
//	GlobalFree( hNewText );

	return 1;
}

void HtmlEntities( LPSTR lpszInput, LPSTR lpszOutput )
{

	if( lpszInput == NULL || lpszOutput == NULL ) {
		Info("Invalid Pointer(s) for HtmlEntities!");
        lstrcpy( lpszOutput, lpszInput );
        return;
    } else lstrcpy( lpszOutput, "" );

    if(lpszInput[0]=='v' && lpszInput[1] == '\0') {
    	char szVersionInfo[1024] = "";
        wsprintf( szVersionInfo,
			"%s 0.1\r\nhttp://www.faireal.net/mion/\r\nBuild: %s %s",
        	lpszAppName, __DATE__, __TIME__
		);
        Info( szVersionInfo );
        lstrcpy( lpszOutput, szVersionInfo );
        return;
    }

	// Default Values to be used when .ini file in unavailable...

	char lpszQuot[128] = "&quot;"; // double quotation to what?
	char lpszApos[128] = "&#39;"; // single quotation to what?
	char lpszBr[128] = "\r\n" ; // new line to <br> ?
	char lpszTab[128] = "    "; // 4 spaces

	InitializeConvParams( lpszQuot, lpszApos, lpszBr, lpszTab );

	int iInputlen = lstrlen( lpszInput );
    int i;
	for(i=0; i < iInputlen; i++) {
    	if(lpszInput[i] == '&') lstrcat(lpszOutput, "&amp;");
    	else if(lpszInput[i] == '<') lstrcat(lpszOutput, "&lt;");
        else if(lpszInput[i] == '>') lstrcat(lpszOutput, "&gt;");
        else if(lpszInput[i] == '"') lstrcat(lpszOutput, lpszQuot );
        else if(lpszInput[i] == '\'') lstrcat(lpszOutput, lpszApos );
        else if(lpszInput[i] == '\r' ) continue;
        else if(lpszInput[i] == '\n' ) lstrcat(lpszOutput, lpszBr);
        else if(lpszInput[i] == '\t') lstrcat(lpszOutput, lpszTab );
        else {
            int last = lstrlen(lpszOutput)-1;
    		lpszOutput[ last+1 ] = lpszInput[ i ];
            lpszOutput[ last+2 ] = '\0';
		}
	}
    
	int iOutputlen = lstrlen( lpszOutput );
    if( iOutputlen > 0 ) {
		char szResult[1024];
        wsprintf( szResult,
        	"Iuput length: %d\r\nOutput length: %d", iInputlen, iOutputlen);
		Info( szResult );
    } else {
    	lstrcpy(lpszOutput, "");
		Info("Unexpected Error!");
	}

}

void Info(LPSTR msg)
{
	MessageBox(NULL, msg, lpszAppName, MB_OK|MB_ICONINFORMATION);
}

int InitializeConvParams(LPSTR lpszQuot, LPSTR lpszApos, LPSTR lpszBr, LPSTR lpszTab)
{
	FILE *fp=fopen("ml-ent.ini", "r");
    if(!fp) return 0;

    char buf[256];
	int i;
    for( i=0; i<100; i++) {
    	if( fgets( buf, sizeof(buf), fp) == NULL ) {
			break;
        }

		if( buf[0] == '#' || buf[0] == '\r' || buf[0] == '\n' ) continue;

		char szVariable[32] = "";
        int iValue = ParseLine( buf, szVariable );

        if( strcmp( szVariable , "quot" ) == 0 ) {
        	iValue == 0 ?
	        	lstrcpy( lpszQuot, "\"" ) : lstrcpy( lpszQuot, "&quot;" ) ;
		} else if( strcmp( szVariable , "apos" ) == 0 ) {
        	iValue == 0 ?
	        	lstrcpy( lpszApos, "'" ) :
            iValue == 1 ?
                lstrcpy( lpszApos, "&apos;" ) : lstrcpy( lpszApos, "&#39;" ) ;
        } else if( strcmp( szVariable, "nl2br") == 0 ) {
			iValue == 0 ?
            	lstrcpy( lpszBr , "\r\n" ) :
            iValue == 1 ?
            	lstrcpy( lpszBr , "<br>\r\n" ) : lstrcpy( lpszBr, "<BR>\r\n" ) ;
        } else if( strcmp( szVariable , "tab" ) == 0 ) {
            if( iValue > 0 && iValue <= MAX_TAB ) {
				int j;
				for( j=0; j<iValue; j++ ) lpszTab[j] = ' '; // tab 2 spaces
				lpszTab[ iValue ] = '\0';
			} else lstrcpy( lpszTab, "\t" );
        }
    }

    fclose(fp);
  	return 1;
}

ParseLine( char *buf, char *szVariable )
{
    char *p1 = szVariable;
	char szValue[32];
    char *p2 = szValue;

    buf[32] = '\0'; // see first 32 bytes only

	int i, iValueFlag = 0;
    for( i=0; i<30; i++ ) {
		if( buf[i]=='\0' ) break;
        else if( buf[i] == ' ' && iValueFlag == 0 ) continue;
        else if( buf[i] == '\t' && iValueFlag == 0 ) continue;
		else if( buf[i] == '=' ) iValueFlag = 1;
		else if( iValueFlag == 0 ) *p1++ = buf[i];
		else *p2++ = buf[i];
	}
    *p1 = '\0', *p2 = '\0';

/*
    char debug[1000];
    sprintf(debug, "%s :\r\n[%s]̒ĺA%d", buf, szVariable, atoi(szValue));
    Info(debug);
*/
	return atoi( szValue );
}

